@@ -165,6 +165,12 @@ keymap = (
     ("", defaultModifierMask, NO_MODIFIER_MASK,
     "increaseSpeechPitchHandler"),
 
+    ("", defaultModifierMask, NO_MODIFIER_MASK,
+    "increaseSpeechVolumeHandler"),
+
+    ("", defaultModifierMask, NO_MODIFIER_MASK,
+    "decreaseSpeechVolumeHandler"),
+
     ("", defaultModifierMask, NO_MODIFIER_MASK,
     "panBrailleLeftHandler"),
 
