@@ -92,7 +92,7 @@ class Script(Gecko.Script):
 
         grid = Gecko.Script.getAppPreferencesGUI(self)
 
-        self.sayAllOnLoadCheckButton.set_active(
+        self._sayAllOnLoadCheckButton.set_active(
             _settingsManager.getSetting('sayAllOnLoad'))
 
         spellcheck = self.spellcheck.getAppPreferencesGUI()
@@ -105,7 +105,7 @@ class Script(Gecko.Script):
         """Returns a dictionary with the app-specific preferences."""
 
         prefs = Gecko.Script.getPreferencesFromGUI(self)
-        prefs['sayAllOnLoad'] = self.sayAllOnLoadCheckButton.get_active()
+        prefs['sayAllOnLoad'] = self._sayAllOnLoadCheckButton.get_active()
         prefs.update(self.spellcheck.getPreferencesFromGUI())
 
         return prefs
@@ -115,9 +115,19 @@ class Script(Gecko.Script):
 
         if self.spellcheck.isActive():
             self.spellcheck.presentErrorDetails(not basicOnly)
+
+        super().doWhereAmI(inputEvent, basicOnly)
+
+    def locusOfFocusChanged(self, event, oldFocus, newFocus):
+        """Handles changes of focus of interest to the script."""
+
+        if self.spellcheck.isSuggestionsItem(newFocus):
+            includeLabel = not self.spellcheck.isSuggestionsItem(oldFocus)
+            self.updateBraille(newFocus)
+            self.spellcheck.presentSuggestionListItem(includeLabel=includeLabel)
             return
 
-        Gecko.Script.doWhereAmI(self,inputEvent, basicOnly)
+        super().locusOfFocusChanged(event, oldFocus, newFocus)
 
     def _useFocusMode(self, obj):
         if self.isEditableMessage(obj):
@@ -153,13 +163,6 @@ class Script(Gecko.Script):
             orca.setLocusOfFocus(event, event.source, False)
             self.updateBraille(orca_state.locusOfFocus)
 
-        if self.spellcheck.isSuggestionsItem(event.source) \
-           and self.spellcheck.isSuggestionsItem(orca_state.locusOfFocus):
-            orca.setLocusOfFocus(event, event.source, False)
-            self.updateBraille(orca_state.locusOfFocus)
-            self.spellcheck.presentSuggestionListItem()
-            return
-
         if not self.utilities.inDocumentContent(obj):
             default.Script.onFocusedChanged(self, event)
             return
