@@ -47,6 +47,9 @@ class Script(web.Script):
             "background-color"        : "bg-color",
             "color"                   : "fg-color"}
 
+        # This one needs some more consideration for all toolkits.
+        self.attributeNamesDict["invalid"] = "text-spelling"
+
     def getUtilities(self):
         """Returns the utilites for this script."""
 
@@ -211,6 +214,16 @@ class Script(web.Script):
         debug.println(debug.LEVEL_INFO, msg)
         default.Script.onNameChanged(self, event)
 
+    def onSelectionChanged(self, event):
+        """Callback for object:selection-changed accessibility events."""
+
+        if super().onSelectionChanged(event):
+            return
+
+        msg = "GECKO: Passing along event to default script"
+        debug.println(debug.LEVEL_INFO, msg)
+        default.Script.onSelectionChanged(self, event)
+
     def onShowingChanged(self, event):
         """Callback for object:state-changed:showing accessibility events."""
 
