@@ -1006,6 +1006,11 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
+        if event.detail1 and self._loadingDocumentContent:
+            msg = "WEB: Ignoring: Already loading document content"
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
         self._loadingDocumentContent = event.detail1
 
         obj, offset = self.utilities.getCaretContext()
@@ -1085,6 +1090,11 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg)
             return False
 
+        if not self.utilities.inDocumentContent(orca_state.locusOfFocus):
+            msg = "WEB: Ignoring event source is in document; locusOfFocus is not"
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
         if self._lastCommandWasCaretNav:
             msg = "WEB: Event ignored: Last command was caret nav"
             debug.println(debug.LEVEL_INFO, msg)
@@ -1164,6 +1174,13 @@ class Script(default.Script):
             self.utilities.setCaretContext(obj, offset)
             return True
 
+        if self.utilities.treatEventAsSpinnerValueChange(event):
+            msg = "WEB: Event handled as the value-change event we wish we'd get"
+            debug.println(debug.LEVEL_INFO, msg)
+            self.updateBraille(event.source)
+            self._presentTextAtNewCaretPosition(event)
+            return True
+
         if not _settingsManager.getSetting('caretNavigationEnabled') \
            or self._inFocusMode or isEditable:
             orca.setLocusOfFocus(event, event.source, False)
@@ -1386,6 +1403,28 @@ class Script(default.Script):
 
         return True
 
+    def onSelectionChanged(self, event):
+        """Callback for object:selection-changed accessibility events."""
+
+        if not self.utilities.inDocumentContent(event.source):
+            msg = "WEB: Event source is not in document content"
+            debug.println(debug.LEVEL_INFO, msg)
+            return False
+
+        if not self.utilities.inDocumentContent(orca_state.locusOfFocus):
+            msg = "WEB: Ignoring event source is in document; locusOfFocus is not"
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
+        obj, offset = self.utilities.getCaretContext()
+        ancestor = self.utilities.commonAncestor(obj, event.source)
+        if ancestor and self.utilities.isTextBlockElement(ancestor):
+            msg = "WEB: Ignoring: Common ancestor of context and event source is text block"
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
+        return False
+
     def onShowingChanged(self, event):
         """Callback for object:state-changed:showing accessibility events."""
 
@@ -1414,6 +1453,11 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
+        if self.utilities.eventIsSpinnerNoise(event):
+            msg = "WEB: Ignoring: Event believed to be spinner noise"
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
         if self.utilities.textEventIsDueToInsertion(event):
             msg = "WEB: Ignoring event believed to be due to text insertion"
             debug.println(debug.LEVEL_INFO, msg)
@@ -1455,6 +1499,11 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
+        if self.utilities.eventIsSpinnerNoise(event):
+            msg = "WEB: Ignoring: Event believed to be spinner noise"
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
         # TODO - JD: As an experiment, we're stopping these at the event manager.
         # If that works, this can be removed.
         if self.utilities.eventIsEOCAdded(event):
