@@ -317,6 +317,12 @@ class Utilities(script_utilities.Utilities):
 
         return super().inFindToolbar(obj)
 
+    def isEmpty(self, obj):
+        if not self.isTextBlockElement(obj):
+            return False
+
+        return self.queryNonEmptyText(obj, False) is None
+
     def isHidden(self, obj):
         try:
             attrs = dict([attr.split(':', 1) for attr in obj.getAttributes()])
@@ -377,6 +383,24 @@ class Utilities(script_utilities.Utilities):
 
         return prevobj, prevoffset
 
+    def lastContext(self, root):
+        offset = 0
+        text = self.queryNonEmptyText(root)
+        if text:
+            offset = text.characterCount - 1
+
+        def _isInRoot(o):
+            return o == root or pyatspi.utils.findAncestor(o, lambda x: x == root)
+
+        obj = root
+        while obj:
+            lastobj, lastoffset = self.nextContext(obj, offset)
+            if not (lastobj and _isInRoot(lastobj)):
+                break
+            obj, offset = lastobj, lastoffset
+
+        return obj, offset
+
     def contextsAreOnSameLine(self, a, b):
         if a == b:
             return True
@@ -549,8 +573,7 @@ class Utilities(script_utilities.Utilities):
                 rv = None
 
         if not self.isLiveRegion(obj):
-            doNotQuery = [pyatspi.ROLE_LIST,
-                          pyatspi.ROLE_TABLE_ROW,
+            doNotQuery = [pyatspi.ROLE_TABLE_ROW,
                           pyatspi.ROLE_TOOL_BAR]
             if rv and obj.getRole() in doNotQuery:
                 rv = None
@@ -1024,6 +1047,10 @@ class Utilities(script_utilities.Utilities):
 
         boundary = pyatspi.TEXT_BOUNDARY_LINE_START
         objects = self._getContentsForObj(obj, offset, boundary)
+        if not layoutMode:
+            if useCache:
+                self._currentLineContents = objects
+            return objects
 
         firstObj, firstStart, firstEnd, firstString = objects[0]
         if extents[2] == 0 and extents[3] == 0:
@@ -1032,23 +1059,6 @@ class Utilities(script_utilities.Utilities):
         lastObj, lastStart, lastEnd, lastString = objects[-1]
         prevObj, pOffset = self.findPreviousCaretInOrder(firstObj, firstStart)
         nextObj, nOffset = self.findNextCaretInOrder(lastObj, lastEnd - 1)
-        if not layoutMode:
-            if firstString and not re.search("\w", firstString) \
-               and (re.match("[^\w\s]", firstString[0]) or not firstString.strip()):
-                onLeft = self._getContentsForObj(prevObj, pOffset, boundary)
-                onLeft = list(filter(_include, onLeft))
-                objects[0:0] = onLeft
-
-            text = self.queryNonEmptyText(nextObj)
-            if text:
-                char = text.getText(nOffset, nOffset + 1)
-                if re.match("[^\w\s]", char):
-                    objects.append([nextObj, nOffset, nOffset + 1, char])
-
-            if useCache:
-                self._currentLineContents = objects
-
-            return objects
 
         # Check for things on the same line to the left of this object.
         while prevObj:
@@ -1140,7 +1150,11 @@ class Utilities(script_utilities.Utilities):
         if not (line and line[0]):
             return []
 
-        lastObj, lastOffset = line[-1][0], line[-1][2] - 1
+        math = self.getMathAncestor(obj)
+        if math:
+            lastObj, lastOffset = self.lastContext(math)
+        else:
+            lastObj, lastOffset = line[-1][0], line[-1][2] - 1
         msg = "INFO: Last context on line is: %s, %i" % (lastObj, lastOffset)
         debug.println(debug.LEVEL_INFO, msg)
 
@@ -1214,6 +1228,7 @@ class Utilities(script_utilities.Utilities):
                              pyatspi.ROLE_FORM,
                              pyatspi.ROLE_HEADING,
                              pyatspi.ROLE_LABEL,
+                             pyatspi.ROLE_LIST,
                              pyatspi.ROLE_LIST_ITEM,
                              pyatspi.ROLE_PANEL,
                              pyatspi.ROLE_PARAGRAPH,
@@ -1841,6 +1856,37 @@ class Utilities(script_utilities.Utilities):
 
         return True
 
+    def isSpinnerEntry(self, obj):
+        if not self.inDocumentContent(obj):
+            return False
+
+        # TODO - JD: Ideally, things that look and act like spinners (such number inputs)
+        # would look and act like platform native spinners. That's not true for Gecko. And
+        # the only thing that's funkier is what we get from WebKitGtk. Try to at least get
+        # the two engines into alignment before migrating Epiphany support to the web script.
+        if obj.getState().contains(pyatspi.STATE_EDITABLE) \
+           and obj.parent.getRole() == pyatspi.ROLE_SPIN_BUTTON:
+            return True
+
+        return False
+
+    def eventIsSpinnerNoise(self, event):
+        if event.type.startswith("object:text-changed") and self.isSpinnerEntry(event.source):
+            lastKey, mods = self.lastKeyAndModifiers()
+            if lastKey in ["Down", "Up"]:
+                return True
+
+        return False
+
+    def treatEventAsSpinnerValueChange(self, event):
+        if event.type.startswith("object:text-caret-moved") and self.isSpinnerEntry(event.source):
+            lastKey, mods = self.lastKeyAndModifiers()
+            if lastKey in ["Down", "Up"]:
+                obj, offset = self.getCaretContext()
+                return event.source == obj
+
+        return False
+
     def eventIsStatusBarNoise(self, event):
         if self.inDocumentContent(event.source):
             return False
@@ -2134,7 +2180,7 @@ class Utilities(script_utilities.Utilities):
                 allText = text.getText(0, -1)
                 for i in range(offset + 1, len(allText)):
                     child = self.getChildAtOffset(obj, i)
-                    if child and not self.isZombie(child):
+                    if child and not self.isZombie(child) and not self.isAnchor(child):
                         return self.findNextCaretInOrder(child, -1)
                     if allText[i] != self.EMBEDDED_OBJECT_CHARACTER:
                         return obj, i
@@ -2186,7 +2232,7 @@ class Utilities(script_utilities.Utilities):
                     offset = len(allText)
                 for i in range(offset - 1, -1, -1):
                     child = self.getChildAtOffset(obj, i)
-                    if child and not self.isZombie(child):
+                    if child and not self.isZombie(child) and not self.isAnchor(child):
                         return self.findPreviousCaretInOrder(child, -1)
                     if allText[i] != self.EMBEDDED_OBJECT_CHARACTER:
                         return obj, i
