@@ -36,7 +36,6 @@ from . import messages
 from . import object_properties
 from . import settings
 from . import settings_manager
-from . import sound
 from . import text_attribute_names
 from . import acss
 
@@ -111,23 +110,6 @@ class SpeechGenerator(generator.Generator):
         """
         generator.Generator._addGlobals(self, globalsDict)
         globalsDict['voice'] = self.voice
-        globalsDict['play'] = self.play
-
-    def play(self, key):
-        """Returns an array containing a sound.Sound instance.
-        The key can a value to be used to look up a filename in the
-        settings.py:sounds dictionary (e.g., a pyatspi.ROLE_* value)
-        or just the name of an audio file to use.
-        """
-        sounds = _settingsManager.getSetting('sounds')
-        try:
-            soundBite = sound.Sound(sounds[key])
-        except:
-            if isinstance(key, str):
-                soundBite = sound.Sound(key)
-            else:
-                soundBite = None
-        return [soundBite]
 
     def generateSpeech(self, obj, **args):
         return self.generate(obj, **args)
@@ -414,7 +396,8 @@ class SpeechGenerator(generator.Generator):
         """
         result = []
         acss = self.voice(DEFAULT)
-        labels = self._script.utilities.unrelatedLabels(obj)
+        visibleOnly = not self._script.utilities.isStatusBarNotification(obj)
+        labels = self._script.utilities.unrelatedLabels(obj, visibleOnly)
         for label in labels:
             name = self._generateName(label, **args)
             result.extend(name)
@@ -803,41 +786,31 @@ class SpeechGenerator(generator.Generator):
         if _settingsManager.getSetting('onlySpeakDisplayedText'):
             return []
 
-        result = []
-        acss = self.voice(STATE)
-        # If this is an icon within an layered pane or a table cell
-        # within a table or a tree table and the item is focused but not
-        # selected, let the user know. See bug #486908 for more details.
-        #
-        checkIfSelected = False
-        objRole, parentRole, state = None, None, None
-        if obj:
-            objRole = obj.getRole()
-            state = obj.getState()
-            if obj.parent:
-                parentRole = obj.parent.getRole()
+        if not obj:
+            return []
 
-        if objRole == pyatspi.ROLE_TABLE_CELL \
-           and (parentRole == pyatspi.ROLE_TREE_TABLE \
-                or parentRole == pyatspi.ROLE_TABLE):
-            checkIfSelected = True
+        if not (obj.parent and 'Selection' in pyatspi.listInterfaces(obj.parent)):
+            return []
 
-        # If we met the last set of conditions, but we got here by
-        # moving left or right on the same row, then don't announce the
-        # selection state to the user. See bug #523235 for more details.
-        #
-        lastKey, mods = self._script.utilities.lastKeyAndModifiers()
-        if checkIfSelected and lastKey in ["Left", "Right"]:
-            checkIfSelected = False
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_SELECTED):
+            return []
 
-        if objRole == pyatspi.ROLE_ICON \
-           and parentRole == pyatspi.ROLE_LAYERED_PANE:
-            checkIfSelected = True
+        parentRole = obj.parent.getRole()
+        if parentRole in [pyatspi.ROLE_TREE_TABLE, pyatspi.ROLE_TABLE]:
+            lastKey, mods = self._script.utilities.lastKeyAndModifiers()
+            if lastKey in ["Left", "Right"]:
+                return []
+            if self._script.utilities.isLayoutOnly(obj.parent):
+                return []
+        elif parentRole == pyatspi.ROLE_LAYERED_PANE:
+            if obj in self._script.utilities.selectedChildren(obj.parent):
+                return []
+        else:
+            return []
 
-        if checkIfSelected \
-           and state and not state.contains(pyatspi.STATE_SELECTED):
-            result.append(object_properties.STATE_UNSELECTED_TABLE_CELL)
-            result.extend(acss)
+        result = [object_properties.STATE_UNSELECTED_TABLE_CELL]
+        result.extend(self.voice(STATE))
 
         return result
 
@@ -2273,7 +2246,8 @@ class SpeechGenerator(generator.Generator):
         return args.get('role', obj.getRole())
 
     def _generatePause(self, obj, **args):
-        if args.get('eliminatePauses', False):
+        if not _settingsManager.getSetting('enablePauseBreaks') \
+           or args.get('eliminatePauses', False):
             return []
         return PAUSE
 
